use std::collections::HashMap;

fn spawn_points(pattern: &str) -> Vec<(i32, i32)> {
    // The points to return, for example R3 will return (0,1),(0,1),(0,1) (three steps to the right)
    let mut pts = vec![];

    // the digits in pattern (3 in 'R3')
    let digits = pattern
        .chars()
        .filter(|x| x.is_numeric())
        .collect::<String>()
        .parse()
        .unwrap();

    // The direction of a pattern ('R' in 'R3')
    let direction: String = pattern.chars().filter(|x| !x.is_numeric()).collect();

    match direction.as_ref() {
        "R" => {
            for _d in 0..digits {
                pts.push((0,1));
            }
        }
        "L" => {
            for _d in 0..digits {
                pts.push((0,-1));
            }
        }
        "U" => {
            for _d in 0..digits {
                pts.push((1,0));
            }
        }
        "D" => {
            for _d in 0..digits {
                pts.push((-1,0));
            }
        }
        _ => {}
    }
    pts

}

fn manhattan_to_origin(pt: (i32,i32)) -> i32 {
    pt.0.abs()+pt.1.abs()
}

fn run_instruction_set(instructions: Vec<String>, grid: &mut HashMap<(i32,i32), i32>) -> Option<i32>{
    let mut cursor = (0,0);
    let mut closest = std::i32::MAX; // largest i32 ever
    for instruction in instructions {
        
        for pt in spawn_points(&instruction) {
            if let Some(occupied_pt) = grid.get(&cursor) {
                println!("Point present @{:?}", cursor);
                let dist = manhattan_to_origin(cursor);
                if dist < closest && dist > 0 {
                    closest = dist;
                }
                grid.insert(cursor, occupied_pt + 1);
            } else {
                grid.insert(cursor, 0);
            }
            cursor.0 += pt.0;
            cursor.1 += pt.1;

        }
    }

    match closest {
        std::i32::MAX => None,
        _ => Some(closest)
    }
}

fn solve() {

    // Hashmap that has a 2d point as key and a vec for wire indices
    let mut grid: HashMap<(i32,i32), i32> = HashMap::new();

    let w1s = include_str!("wire1");
    let w2s = include_str!("wire2");

    let w1: Vec<String> = w1s.split(",").map(|s| s.to_string()).collect();
    let w2: Vec<String> = w2s.split(",").map(|s| s.to_string()).collect();

    run_instruction_set(w1, &mut grid);
    dbg!(run_instruction_set(w2, &mut grid));


}


pub fn main() {
    solve();
}